"""
mbed SDK
Copyright (c) 2011-2016 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Przemyslaw Wirkus <Przemyslaw.Wirkus@arm.com>
"""

from __future__ import print_function
from multiprocessing import freeze_support
from mbed_os_tools.test import init_host_test_cli_params
from mbed_os_tools.test.host_tests_runner.host_test_default import DefaultTestSelector
from mbed_os_tools.test.host_tests_toolbox.host_functional import handle_send_break_cmd


def main():
    """! This function drives command line tool 'mbedhtrun' which is using DefaultTestSelector
    @details 1. Create DefaultTestSelector object and pass command line parameters
             2. Call default test execution function run() to start test instrumentation
    """
    freeze_support()
    result = 0
    cli_params = init_host_test_cli_params()

    if cli_params.version:         # --version
        import pkg_resources    # part of setuptools
        version = pkg_resources.require("mbed-host-tests")[0].version
        print(version)
    elif cli_params.send_break_cmd:  # -b with -p PORT (and optional -r RESET_TYPE)
        handle_send_break_cmd(
            port=cli_params.port,
            disk=cli_params.disk,
            reset_type=cli_params.forced_reset_type,
            baudrate=cli_params.baud_rate,
            verbose=cli_params.verbose
        )
    else:
        test_selector = DefaultTestSelector(cli_params)
        try:
            result = test_selector.execute()
            # Ensure we don't return a negative value
            if result < 0 or result > 255:
                result = 1
        except (KeyboardInterrupt, SystemExit):
            test_selector.finish()
            result = 1
            raise
        else:
            test_selector.finish()

    return result



"""
mbed SDK
Copyright (c) 2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os

from mbed_host_tests.host_tests_plugins.module_copy_mps2 import HostTestPluginCopyMethod_MPS2

class MPS2CopyTestCase(unittest.TestCase):

    def setUp(self):
        self.mps2_copy_plugin = HostTestPluginCopyMethod_MPS2()
        self.filename = "toto.bin"
        # Create the empty file named self.filename
        open(self.filename, "w+").close()

    def tearDown(self):
        os.remove(self.filename)

    def test_copy_bin(self):
        # Check that file has been copied as "mbed.bin"
        self.mps2_copy_plugin.mps2_copy(self.filename, ".")
        self.assertTrue(os.path.isfile("mbed.bin"))
        os.remove("mbed.bin")

    def test_copy_elf(self):
        # Check that file has been copied as "mbed.elf"
        os.rename(self.filename, "toto.elf")
        self.filename = "toto.elf"
        self.mps2_copy_plugin.mps2_copy(self.filename, ".")
        self.assertTrue(os.path.isfile("mbed.elf"))
        os.remove("mbed.elf")

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
from mbed_host_tests.host_tests_registry import HostRegistry
from mbed_host_tests import BaseHostTest


class HostRegistryTestCase(unittest.TestCase):

    class HostTestClassMock(BaseHostTest):
        def setup(self):
            pass

        def result(self):
            pass

        def teardown(self):
            pass

    def setUp(self):
        self.HOSTREGISTRY = HostRegistry()

    def tearDown(self):
        pass

    def test_register_host_test(self):
        self.HOSTREGISTRY.register_host_test('host_test_mock_auto', self.HostTestClassMock())
        self.assertEqual(True, self.HOSTREGISTRY.is_host_test('host_test_mock_auto'))

    def test_unregister_host_test(self):
        self.HOSTREGISTRY.register_host_test('host_test_mock_2_auto', self.HostTestClassMock())
        self.assertEqual(True, self.HOSTREGISTRY.is_host_test('host_test_mock_2_auto'))
        self.assertNotEqual(None, self.HOSTREGISTRY.get_host_test('host_test_mock_2_auto'))
        self.HOSTREGISTRY.unregister_host_test('host_test_mock_2_auto')
        self.assertEqual(False, self.HOSTREGISTRY.is_host_test('host_test_mock_2_auto'))

    def test_get_host_test(self):
        self.HOSTREGISTRY.register_host_test('host_test_mock_3_auto', self.HostTestClassMock())
        self.assertEqual(True, self.HOSTREGISTRY.is_host_test('host_test_mock_3_auto'))
        self.assertNotEqual(None, self.HOSTREGISTRY.get_host_test('host_test_mock_3_auto'))

    def test_is_host_test(self):
        self.assertEqual(False, self.HOSTREGISTRY.is_host_test(''))
        self.assertEqual(False, self.HOSTREGISTRY.is_host_test(None))
        self.assertEqual(False, self.HOSTREGISTRY.is_host_test('xyz'))

    def test_host_test_str_not_empty(self):
        for ht_name in self.HOSTREGISTRY.HOST_TESTS:
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            self.assertNotEqual(None, ht)

    def test_host_test_has_name_attribute(self):
        for ht_name in self.HOSTREGISTRY.HOST_TESTS:
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            self.assertTrue(hasattr(ht, 'setup'))
            self.assertTrue(hasattr(ht, 'result'))
            self.assertTrue(hasattr(ht, 'teardown'))


if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from mbed_host_tests.host_tests_conn_proxy.conn_primitive_remote import RemoteConnectorPrimitive


class RemoteResourceMock(object):
    def __init__(self, requirements):
        self._is_allocated = True
        self._is_connected = True
        self.requirements = requirements
        self.open_connection = mock.MagicMock()
        self.close_connection = mock.MagicMock()
        self.write = mock.MagicMock()
        self.read = mock.MagicMock()
        self.read.return_value = "abc"
        self.disconnect = mock.MagicMock()
        self.flash = mock.MagicMock()
        self.reset = mock.MagicMock()
        self.release = mock.MagicMock()

    @property
    def is_connected(self):
        return self._is_connected

    @property
    def is_allocated(self):
        return self._is_allocated


class RemoteModuleMock(object):
    class SerialParameters(object):
        def __init__(self, baudrate):
            self.baudrate = baudrate

    def __init__(self, host, port):
        self.host = host
        self.port = port
        self.is_allocated_mock = mock.MagicMock()
        self.allocate = mock.MagicMock()
        self.allocate.side_effect = lambda req: RemoteResourceMock(req)
        self.get_resources = mock.MagicMock()
        self.get_resources.return_value = [1]

    @staticmethod
    def create(host, port):
        return RemoteModuleMock(host, port)


class ConnPrimitiveRemoteTestCase(unittest.TestCase):

    def setUp(self):
        self.config = {
            "grm_module": "RemoteModuleMock",
            "tags": "a,b",
            "image_path": "test.bin",
            "platform_name": "my_platform",
        }
        self.importer = mock.MagicMock()
        self.importer.side_effect = lambda x: RemoteModuleMock
        self.remote = RemoteConnectorPrimitive("remote", self.config, self.importer)

    def test_constructor(self):
        self.importer.assert_called_once_with("RemoteModuleMock")

        self.remote.client.get_resources.called_once()
        self.assertEqual(self.remote.remote_module, RemoteModuleMock)
        self.assertIsInstance(self.remote.client, RemoteModuleMock)
        self.assertIsInstance(self.remote.selected_resource, RemoteResourceMock)

        # allocate is called
        self.remote.client.allocate.assert_called_once_with({
            'platform_name': self.config.get('platform_name'),
            'tags': {"a": True, "b": True}})

        # flash is called
        self.remote.selected_resource.open_connection.called_once_with("test.bin")

        # open_connection is called
        self.remote.selected_resource.open_connection.called_once()
        connect = self.remote.selected_resource.open_connection.call_args[1]
        self.assertEqual(connect["parameters"].baudrate, 9600)

        # reset once
        self.remote.selected_resource.reset.assert_called_once_with()

    def test_write(self):
        self.remote.write("abc")
        self.remote.selected_resource.write.assert_called_once_with("abc")

    def test_read(self):
        data = self.remote.read(6)
        self.remote.selected_resource.read.assert_called_once_with(6)
        self.assertEqual(data, "abc")

    def test_reset(self):
        self.remote.reset()
        self.assertEqual(self.remote.selected_resource.reset.call_count, 2)

    def test_finish(self):
        resource = self.remote.selected_resource
        self.remote.finish()
        self.assertEqual(self.remote.selected_resource, None)
        resource.close_connection.assert_called_once()
        resource.release.assert_called_once()


if __name__ == '__main__':
    unittest.main()



# Copyright 2015 ARM Limited, All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from mbed_host_tests.host_tests.base_host_test import BaseHostTest, event_callback


class TestEvenCallbackDecorator(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_event_callback_decorator(self):
        class Ht(BaseHostTest):

            @event_callback('Hi')
            def hi(self, key, value, timestamp):
                print('hi')

            @event_callback('Hello')
            def hello(self, key, value, timestamp):
                print('hello')
        h = Ht()
        h.setup()
        callbacks = h.get_callbacks()
        self.assertIn('Hi', callbacks)
        self.assertIn('Hello', callbacks)



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

from mbed_host_tests.host_tests_registry import HostRegistry

class BaseHostTestTestCase(unittest.TestCase):

    def setUp(self):
        self.HOSTREGISTRY = HostRegistry()

    def tearDown(self):
        pass

    def test_host_test_has_setup_teardown_attribute(self):
        for ht_name in self.HOSTREGISTRY.HOST_TESTS:
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            self.assertTrue(hasattr(ht, 'setup'))
            self.assertTrue(hasattr(ht, 'teardown'))

    def test_host_test_has_no_rampUpDown_attribute(self):
        for ht_name in self.HOSTREGISTRY.HOST_TESTS:
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            self.assertFalse(hasattr(ht, 'rampUp'))
            self.assertFalse(hasattr(ht, 'rampDown'))

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2016 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Przemyslaw Wirkus <Przemyslaw.Wirkus@arm.com>
"""

import sys
import optparse

### Flashing/Reset API provided by mbed--host-tests (mbedhtrun)
from mbed_os_tools.test import host_tests_plugins


def cmd_parser_setup():
    """! Creates simple command line parser
    """
    parser = optparse.OptionParser()

    parser.add_option('-f', '--file',
                      dest='filename',
                      help='File to flash onto mbed device')

    parser.add_option("-d", "--disk",
                      dest="disk",
                      help="Target disk (mount point) path. Example: F:, /mnt/MBED",
                      metavar="DISK_PATH")

    copy_methods_str = "Plugin support: " + ', '.join(host_tests_plugins.get_plugin_caps('CopyMethod'))

    parser.add_option("-c", "--copy",
                      dest="copy_method",
                      default='shell',
                      help="Copy (flash the target) method selector. " + copy_methods_str,
                      metavar="COPY_METHOD")

    parser.add_option('', '--plugins',
                      dest='list_plugins',
                      default=False,
                      action="store_true",
                      help='Prints registered plugins and exits')

    parser.add_option('', '--version',
                      dest='version',
                      default=False,
                      action="store_true",
                      help='Prints package version and exits')

    parser.description = """Flash mbed devices from command line.""" \
        """This module is using build in to mbed-host-tests plugins used for flashing mbed devices"""
    parser.epilog = """Example: mbedflsh -d E: -f /path/to/file.bin"""

    (opts, args) = parser.parse_args()
    return (opts, args)


def main():
    """! Function wrapping flashing (copy) plugin
    @details USers can use mbedflsh command to flash mbeds from command line
    """
    errorlevel_flag = 0
    (opts, args) = cmd_parser_setup()

    if opts.version:
        import pkg_resources  # part of setuptools
        version = pkg_resources.require("mbed-host-tests")[0].version
        print(version)
        sys.exit(0)
    elif opts.list_plugins:    # --plugins option
        host_tests_plugins.print_plugin_info()
        sys.exit(0)
    else:
        pass

    if opts.filename:
        print("mbedflsh: opening file %s..."% opts.filename)
        result = host_tests_plugins.call_plugin('CopyMethod',
            opts.copy_method,
            image_path=opts.filename,
            destination_disk=opts.disk)
        errorlevel_flag = result == True

    return errorlevel_flag



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

import os
import re
import sys
import platform
from mbed_host_tests.host_tests_plugins.host_test_plugins import HostTestPluginBase


class HostOSDetectionTestCase(unittest.TestCase):

    def setUp(self):
        self.plugin_base = HostTestPluginBase()
        self.os_names = ['Windows7', 'Ubuntu', 'LinuxGeneric', 'Darwin']
        self.re_float = re.compile("^\d+\.\d+$")

    def tearDown(self):
        pass

    def test_os_info(self):
        self.assertNotEqual(None, self.plugin_base.mbed_os_info())

    def test_os_support(self):
        self.assertNotEqual(None, self.plugin_base.mbed_os_support())

    def test_supported_os_name(self):
        self.assertIn(self.plugin_base.mbed_os_support(), self.os_names)

    def test_detect_os_support_ext(self):
        os_info = (os.name,
                   platform.system(),
                   platform.release(),
                   platform.version(),
                   sys.platform)

        self.assertEqual(os_info, self.plugin_base.mbed_os_info())


if __name__ == '__main__':
    unittest.main()



"""
This module defines the attributes of the
PyPI package for the mbed SDK test suite ecosystem tools
"""

"""
mbed SDK
Copyright (c) 2011-2016 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Przemyslaw Wirkus <Przemyslaw.Wirkus@arm.com>
"""

import os
from distutils.core import setup
from io import open
from setuptools import find_packages

DESCRIPTION = "mbed tools used to flash, reset and supervise test execution for mbed-enabled devices"
OWNER_NAMES = 'Jimmy Brisson, Brian Daniels'
OWNER_EMAILS = 'Jimmy.Brisson@arm.com, Brian.Daniels@arm.com'


# Utility function to cat in a file (used for the README)
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), encoding="utf8").read()

setup(name='mbed-host-tests',
      version='1.5.3',
      description=DESCRIPTION,
      long_description=read('README.md'),
      long_description_content_type='text/markdown',
      author=OWNER_NAMES,
      author_email=OWNER_EMAILS,
      maintainer=OWNER_NAMES,
      maintainer_email=OWNER_EMAILS,
      url='https://github.com/ARMmbed/mbed-os-tools',
      packages=find_packages(),
      license="Apache-2.0",
      test_suite = 'test',
      entry_points={
        "console_scripts":
            ["mbedhtrun=mbed_host_tests.mbedhtrun:main",
             "mbedflsh=mbed_host_tests.mbedflsh:main"],
      },
      install_requires=[
        "mbed-os-tools==0.0.4"
      ],
      tests_require=[
        "mock>=2"
      ]
)



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import six
import unittest
from mbed_host_tests.host_tests_registry import HostRegistry


class HostRegistryTestCase(unittest.TestCase):

    def setUp(self):
        self.HOSTREGISTRY = HostRegistry()

    def tearDown(self):
        pass

    def test_host_test_class_has_test_attr(self):
        """ Check if host test has 'result' class member
        """
        for i, ht_name in enumerate(self.HOSTREGISTRY.HOST_TESTS):
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            if ht is not None:
                self.assertEqual(True, hasattr(ht, 'result'))

    def test_host_test_class_test_attr_callable(self):
        """ Check if host test has callable 'result' class member
        """
        for i, ht_name in enumerate(self.HOSTREGISTRY.HOST_TESTS):
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            if ht:
                self.assertEqual(True, hasattr(ht, 'result') and callable(getattr(ht, 'result')))

    def test_host_test_class_test_attr_callable_args_num(self):
        """ Check if host test has callable setup(), result() and teardown() class member has 2 arguments
        """
        for i, ht_name in enumerate(self.HOSTREGISTRY.HOST_TESTS):
            ht = self.HOSTREGISTRY.HOST_TESTS[ht_name]
            if ht and hasattr(ht, 'setup') and callable(getattr(ht, 'setup')):
                self.assertEqual(1, six.get_function_code(ht.setup).co_argcount)
            if ht and hasattr(ht, 'result') and callable(getattr(ht, 'result')):
                self.assertEqual(1, six.get_function_code(ht.result).co_argcount)
            if ht and hasattr(ht, 'teardown') and callable(getattr(ht, 'teardown')):
                self.assertEqual(1, six.get_function_code(ht.teardown).co_argcount)


if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Przemyslaw Wirkus <Przemyslaw.Wirkus@arm.com>
"""

from mbed_os_tools.test.host_tests_plugins.module_copy_mps2 import (
    HostTestPluginCopyMethod_MPS2,
    load_plugin,
)



"""
mbed SDK
Copyright (c) 2018 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock
import os
import time

from mbed_host_tests.host_tests_plugins.module_reset_mps2 import HostTestPluginResetMethod_MPS2

class MPS2ResetTestCase(unittest.TestCase):

    def setUp(self):
        self.mps2_reset_plugin = HostTestPluginResetMethod_MPS2()

    def tearDown(self):
        pass

    @mock.patch("os.name", "posix")
    @mock.patch("time.sleep")
    @mock.patch("mbed_host_tests.host_tests_plugins.module_reset_mps2.HostTestPluginResetMethod_MPS2.run_command")
    def test_check_sync(self, run_command_function, sleep_function):
        # Check that a sync call has correctly been executed
        self.mps2_reset_plugin.execute("reboot.txt", disk=".")
        args, _ = run_command_function.call_args
        self.assertTrue("sync" in args[0])
        os.remove("reboot.txt")

if __name__ == '__main__':
    unittest.main()
